/**
 * Class Plotdata
 * 
 * Stores data for one 2D plotting axis
 *
 * Provides operators and mathematical functions to map a Plotdata
 * object to another using unary and binary functions.
 *
 * Provides insertion and extraction of special commands into and from
 * Plotdata pairs as static operations.
 * Available commands are:
 *      colourChange (to change the foreground colour of graphs)
 *      singlePoint  (to draw a single point of data as a special marker)
 *
 * Author: 	jlk
 * Version:	1.1
 * Date:	July 2005
 */
 
#pragma once

#include <cmath>
#include <vector>
#include <iostream>
#include <limits>

// BEGIN Windows-specific, MSVC2008 specific
#include <windows.h>
typedef COLORREF Color;	// central switch between GDI and GDIplus
#define COLOR(r,g,b) RGB(r,g,b)

const Color BLACK	=RGB( 0, 0, 0 );
const Color BLUE	=RGB( 0, 0, 128);        
const Color GREEN	=RGB( 0, 128, 0 );       
const Color CYAN	=RGB( 0, 128, 128 );     
const Color RED		=RGB( 128, 0, 0 );       
const Color MAGENTA	=RGB( 128, 0, 128 );     
const Color BROWN	=RGB( 128, 128, 0 );     
const Color LIGHTGRAY	=RGB( 192, 192, 192 );   
const Color DARKGRAY	=RGB( 128, 128, 128 );   
const Color LIGHTBLUE	=RGB( 128, 128, 255 );   
const Color LIGHTGREEN	=RGB( 128, 255, 128 );   
const Color LIGHTCYAN	=RGB( 128, 255, 255 );   
const Color LIGHTRED	=RGB( 255, 128, 128 );   
const Color LIGHTMAGENTA=RGB( 255, 128, 255 );   
const Color YELLOW	=RGB( 255, 255, 0 );     
const Color WHITE	=RGB( 255, 255, 255 );   

const Color CRIMSON		=COLOR(220, 20, 60);
const Color CHOCOLATE		=COLOR(210, 105, 30);
const Color GOLDENROD		=COLOR(218, 165, 32);
const Color DARKORANGE		=COLOR(255, 140, 0);
const Color REDRED		=COLOR(255, 0, 0);
const Color DARKSLATEGRAY	=COLOR(47, 79, 79);
const Color DEEPPINK		=COLOR(255, 20, 147);
const Color TURQUOISE		=COLOR(64, 224, 208);
const Color DARKVIOLET		=COLOR(148, 0, 211);
const Color BEIGE		=COLOR(245, 245, 220);
const Color MEDIUMSEAGREEN	=COLOR(60, 179, 113);
const Color LIMEGREEN		=COLOR(50, 205, 50);
const Color DARKGREEN		=COLOR(0, 100, 0);
const Color MEDIUMBLUE		=COLOR(0, 0, 205);
const Color BLUEBLUE		=COLOR(0, 0, 255);
const Color DODGERBLUE		=COLOR(30, 144, 255);

/* -------------------------------------------------------- */

namespace std{
 bool isfinite(double);
 double round(double);
 double remquo(double,double,int*);
 double trunc(double);
}
// END Windows-specific, MSVC2008 specific
typedef double float_t;

using namespace std;

// Unary function pointer type accepted by Plotdata (double parameter)
typedef float_t (*Func)(float_t operand);

// Binary function pointer type accepted by Plotdata
typedef float_t (*BinFunc)(float_t operand1, float_t operand2);

/** When true defines a range to be logarithmic */
typedef bool LogSpace;

/** Define data iterator */
typedef vector<float_t>::const_iterator  dataIterator;

/** Remove case sensitivity for Plotdata class */
class Plotdata;

/** #define Plotdata & as a type (avoid using confusing '&' symbols in C) */

/** Number of points in a plot */
enum Grain
{
	EXTRA_FINE = 901,
	FINE = 501,
	MEDIUM = 301,
	COARSE = 201,
	GROSS = 101
};

const double NOPLOT = numeric_limits<double>::quiet_NaN(); // Any point of this value will not be plotted

const int NOCOLOR      = -1;
const int RESETCOLOR   = -2;

class Plotdata{
    // --------------------------------------------------------------
    //     OPERATORS
        
 friend ostream & operator << (ostream & out, const Plotdata & pd);
 friend istream & operator >> (istream & in, Plotdata & pd);
 friend Plotdata operator + (float_t op1, const Plotdata & pd);
 friend Plotdata operator - (float_t op1, const Plotdata & pd);
 friend Plotdata operator * (float_t op1, const Plotdata & pd);
 friend Plotdata operator / (float_t op1, const Plotdata & pd);
	      
public:

 Plotdata operator +  (const Plotdata &) const;
 Plotdata operator +  (float_t) const;
 Plotdata operator -  (const Plotdata &) const;
 Plotdata operator -  (float_t) const;
 Plotdata operator *  (const Plotdata &) const;
 Plotdata operator *  (float_t) const; 
 Plotdata operator /  (const Plotdata &) const;
 Plotdata operator /  (float_t) const;
 Plotdata operator ^  (float_t) const;
 Plotdata operator -  ( ) const;
 Plotdata & operator << (const Plotdata &); // concatenate
 Plotdata & operator << (float_t); // add a double to the data

    // --------------------------------------------------------------
    // Constructors
 inline Plotdata(): data(MEDIUM), userFunction(0),userBinFunction(0){}
 Plotdata(float_t min, float_t max, Grain grain=MEDIUM);
 Plotdata(const float_t*array, int dataSize);
 inline Plotdata(size_t s): data(s), userFunction(0),userBinFunction(0){}
 inline Plotdata(vector<float_t> d): data(d),userFunction(0), userBinFunction(0){};
    // Member Functions
 void insert(const float_t array[], int dataSize);
 inline size_t size() const {return data.size();}
 inline void point(float_t p) {data.push_back(p);}
 void plotRange(float_t min,float_t max,size_t numPoints,bool isLog = false);

 float_t minY()const;
 float_t maxY()const;
 inline void clear() {data.clear();}
 inline const vector<float_t> & getData() const{return data;}
 inline Func & userfunc() { return userFunction; }
 inline BinFunc & userBinfunc() { return userBinFunction; }

 Plotdata doFunc(Func aFunction) const;
 Plotdata doBinFunc(BinFunc aFunction, float_t operand2) const;
 Plotdata doBinFunc(float_t operand2) const;

    // --------------------------------------------------------------
    // Class (static) functions
    
    // Retrieves the maximum of each of x and y in a Plotdata pair
    // Values corresponding to a NAN in the other Plotdata are not included
 static void maxXY(const Plotdata &x, const Plotdata &y,
                    float_t &xMax, float_t &yMax);
    // Retrieves the minimum of each of x and y in a Plotdata pair
    // Values corresponding to a NAN in the other Plotdata are not included
 static void minXY(const Plotdata &x, const Plotdata &y,
                    float_t &xMin, float_t &yMin);
    // Retrieve the single point coordinates requested by special command
 static bool singlePoint(float_t &xCoord, float_t &yCoord, 
                            dataIterator &x, dataIterator &y);
    // Insert singlePoint coordinates into plot data pair
 static void singlePoint(Plotdata &x,Plotdata &y,float_t xCoord,float_t yCoord);
    // Retrieve the colour change requested by special command
 static Color colorChange(dataIterator &x, dataIterator &y);
    // Insert special command for a colour change
 static void colorChange(Plotdata &x, Plotdata &y, int colour);
    // Insert special command to reset colour to before last change
 static void colorReset(Plotdata &x, Plotdata &y);
    // Return true if theColour is a valid colour or RESETCOLOR
 static bool isColor(Color theColour);
    // Insert a single (invisible) point in plotdata pair
 static void soliton(Plotdata &x, Plotdata &y, float_t xval, float_t yval );
    
private:
 vector<float_t> data;
 Func userFunction;		// Any user-designed unary function
 BinFunc userBinFunction;	// Any user-designed binary function
};
